/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import java.io.Serializable;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;


/**
 * <p>
 * Defines the contract for a node matcher. Implementations need to provide three functions:
 * </p>
 * <ol>
 * <li>A matching function that will take a node and determine if it matches</li>
 * <li>A default title string used for those nodes that match</li>
 * <li>A default url to use if no contextual information is available</li>
 * </ol>
 *
 * <p>
 * Note, implementations of this interface <b>must</b> override the equals/hashCode methods because the
 * implementation will be stored in a map (likely a hash map).
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations of this interface should be thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
public interface NodeMatcher extends Serializable {
    /**
     * <p>
     * Determines whether the specified node matches this node matcher.</p>
     *
     * @param node A non-null node to test
     * @return true if the node matches, false otherwise
     *
     * @throws IllegalArgumentException if the node is not null
     */
    public boolean matches(BreadCrumbNode node);

    /**
     * <p>
     * Returns the default title that nodes that match this node matcher can use if their title is not specified.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the default title
     */
    public String getTitle();

    /**
     * <p>
     * Returns the default url that can be used if contextual node information is not available.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the url
     */
    public String getUrl();
}
