/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for BreadCrumbNode.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbNodeTests extends TestCase {
    /**
     * <p>
     * Title string for creating new BreadCrumbNode instance.
     * </p>
     *
     */
    private static final String TITLE = "TOPCODER";

    /**
     * <p>
     * URL String for creating new BreadCrumbNode instance.
     * </p>
     *
     */
    private static final String URL = "http://www.topcoder.com";

    /**
     * <p>
     * BreadCrumbNode instance for testing.
     * </p>
     *
     */
    private BreadCrumbNode node;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        node = new BreadCrumbNode(URL, TITLE);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        node = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(BreadCrumbNodeTests.class);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbNode#BreadCrumbNode(String,String) for accuracy.
     * Expects node is created successfully.
     * </p>
     *
     */
    public void testCtor() {
        assertNotNull("Failed to create new BreadCrumbNode instance.", node);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbNode#BreadCrumbNode(String,String) for failure.
     * It tests the case that when url is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_NullUrl() {
        try {
            new BreadCrumbNode(null, TITLE);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor BreadCrumbNode#BreadCrumbNode(String,String) for failure.
     * It tests the case that when url is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_EmptyUrl() {
        try {
            new BreadCrumbNode(" ", TITLE);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor BreadCrumbNode#BreadCrumbNode(String,String) for accuracy.
     * It tests the case that when title is null and expects success.
     * </p>
     */
    public void testCtor1_NullTitle() {
        node = new BreadCrumbNode(URL, null);

        assertNull("Expects title is null.", node.getTitle());
    }

    /**
     * <p>
     * Tests ctor BreadCrumbNode#BreadCrumbNode(String,String) for accuracy.
     * It tests the case that when title is empty and expects success.
     * </p>
     */
    public void testCtor1_EmptyTitle() {
        node = new BreadCrumbNode(URL, "  ");

        assertNull("Expects title is null.", node.getTitle());
    }

    /**
     * <p>
     * Tests BreadCrumbNode#getTitle() for accuracy.
     * Expects title is correctly set.
     * </p>
     */
    public void testGetTitle() {
        assertEquals("Failed to get correct title.", TITLE, node.getTitle());
    }

    /**
     * <p>
     * Tests BreadCrumbNode#getUrl() for accuracy.
     * Expects url is correctly set.
     * </p>
     */
    public void testGetUrl() {
        assertEquals("Failed to get correct url.", URL, node.getUrl());
    }

}