/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTag;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for BreadCrumbTrailTag.
 * </p>
 *
 * <p>
 * For in-container test cases, please see BreadCrumbTrailTagInContainerTests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbTrailTagTests extends TestCase {
    /**
     * <p>
     * STYLE setting for testing.
     * </p>
     */
    private static final String STYLE = "color:red";

    /**
     * <p>
     * BreadCrumbTrailTag instance for testing.
     * </p>
     */
    private BreadCrumbTrailTag tag;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        tag = new BreadCrumbTrailTag();
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        tag = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(BreadCrumbTrailTagTests.class);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when no namespace is configed.
     * </p>
     */
    public void testCtor_NoConfiged() {
        assertNotNull("Failed to create new BreadCrumbTrailTag instance.", tag);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and pathSeparator is configed.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtro_PathSeparatorConfiged() throws Exception {
        String configFile = "test_files/config/PathSeparator.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct pathSeparator", "-", tag.getPathSeparator());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and pathSeparatorStyle is configed.
     * The pathSeparatorStyle value is non-empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_NonEmptyPathSeparatorStyleConfiged() throws Exception {
        String configFile = "test_files/config/PathSeparatorStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct pathSeparatorStyle", STYLE, tag.getPathSeparatorStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and pathSeparatorStyle is configed.
     * The pathSeparatorStyle value is empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_EmptyPathSeparatorStyleConfiged() throws Exception {
        String configFile = "test_files/config/EmptyPathSeparatorStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertNull("Expects null path separator style.", tag.getPathSeparatorStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and nodeStyle is configed.
     * The nodeStyle value is non-empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_NonEmptyNodeStyleConfiged() throws Exception {
        String configFile = "test_files/config/NodeStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct nodeStyle", STYLE, tag.getNodeStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and nodeStyle is configed.
     * The nodeStyle value is empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_EmptyNodeStyleConfiged() throws Exception {
        String configFile = "test_files/config/EmptyNodeStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertNull("Expects null node style.", tag.getNodeStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and rootNodeStyle is configed.
     * The rootNodeStyle value is non-empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_NonEmptyRootNodeStyleConfiged() throws Exception {
        String configFile = "test_files/config/RootNodeStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct root node style.", STYLE, tag.getRootNodeStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and rootNodeStyle is configed.
     * The rootNodeStyle value is empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_EmptyRootNodeStyleConfiged() throws Exception {
        String configFile = "test_files/config/EmptyRootNodeStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertNull("Expects null root node style.", tag.getRootNodeStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and currentNodeStyle is configed.
     * The currentNodeStyle value is non-empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_NonEmptyCurrentNodeStyleConfiged() throws Exception {
        String configFile = "test_files/config/CurrentNodeStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct current node style.", STYLE, tag.getCurrentNodeStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and currentNodeStyle is configed.
     * The currentNodeStyle value is empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_EmptyCurrentNodeStyleConfiged() throws Exception {
        String configFile = "test_files/config/EmptyCurrentNodeStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertNull("Expects null current node style.", tag.getCurrentNodeStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and mouseOverStyle is configed.
     * The mouseOverStyle value is non-empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_NonEmptyMouseOverStyleConfiged() throws Exception {
        String configFile = "test_files/config/MouseOverStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct mouse over style.", STYLE, tag.getMouseOverStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and mouseOverStyle is configed.
     * The mouseOverStyle value is empty.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_EmptyMouseOverStyleConfiged() throws Exception {
        String configFile = "test_files/config/EmptyMouseOverStyle.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertNull("Expects null mouse over style.", tag.getMouseOverStyle());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and pathDirection is configed.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_PathDirectionConfiged() throws Exception {
        String configFile = "test_files/config/PathDirection.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct path direction", 0, tag.getPathDirection());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and pathDirection is configed.
     * The pathDirection value is not valid and so default direction value will be used.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_InvalidPathDirectionConfiged() throws Exception {
        String configFile = "test_files/config/InvalidPathDirection.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertEquals("Failed to get correct path direction", 0, tag.getPathDirection());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor BreadCrumbTrailTag#BreadCrumbTrailTag() for accuracy.
     * Expects tag is successfully created when namespace is configed and silentErrors is configed.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor_SilentErrosConfiged() throws Exception {
        String configFile = "test_files/config/SilentErrors.xml";
        TestHelper.loadSingleXMLConfig(TestHelper.NAMESPACE, configFile);

        tag = new BreadCrumbTrailTag();

        assertTrue("Failed to get correct silentErrors", tag.getSilentErrors());

        TestHelper.clearConfigFile(TestHelper.NAMESPACE);
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getPathSeparator() for accuracy.
     * Expects the default value for pathSeparator is <b>:</b>.
     * </p>
     */
    public void testGetPathSeparator() {
        assertEquals("Default pathSeparator should be ':'.", ":", tag.getPathSeparator());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathSeparator(String) for accuracy.
     * It tests the case when pathSeparator is empty and expects it's correctly set.
     * </p>
     */
    public void testSetPathSeparator() {
        tag.setPathSeparator(" ");

        assertEquals("Failed to get correct path separator.", " ", tag.getPathSeparator());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathSeparator(String) for failure.
     * It tests the case that when pathSeparator is null and expects IllegalArgumentException.
     * </p>
     */
    public void testSetPathSeparator_NullPathSeparator() {
        try {
            tag.setPathSeparator(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getTitle() for accuracy.
     * Expects the default value for title is null.
     * </p>
     */
    public void testGetTitle() {
        assertNull("Expect title is initially null.", tag.getTitle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setTitle(String) for accuracy.
     * Expects the title is correctly set.
     * </p>
     */
    public void testSetTitle() {
        tag.setTitle("Main");

        assertEquals("Failed to get correct title.", "Main", tag.getTitle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setTitle(String) for accuracy.
     * It tests the case that when title is null and expects success.
     * </p>
     */
    public void testSetTitle_NullTitle() {
        tag.setTitle(null);

        assertNull("Expect title is null.", tag.getTitle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setTitle(String) for accuracy.
     * It tests the case that when title is empty and expects success.
     * </p>
     */
    public void testSetTitle_EmptyTitle() {
        tag.setTitle(" ");

        assertNull("Expect title is null.", tag.getTitle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getPathDirection() for accuracy.
     * Expects the default value for pathDirection is zero.
     * </p>
     */
    public void testGetPathDirection() {
        assertEquals("Default pathDirection should be 0.", BreadCrumbTrailTag.ROOT_TO_CURRENT, tag.getPathDirection());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathDirection(int) for accuracy.
     * Expects pathDirection is correctly set.
     * </p>
     */
    public void testSetPathDirection() {
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        assertEquals("Failed to get correct path direction.", BreadCrumbTrailTag.CURRENT_TO_ROOT,
                tag.getPathDirection());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathDirection(int) for failure.
     * It tests the case that when pathDirection is invalid and expects IllegalArgumentException.
     * </p>
     */
    public void testSetPathSeparator_InvalidValue() {
        try {
            tag.setPathDirection(444);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getPathSeparatorStyle() for accuracy.
     * Expects the default value for pathSeparatorStyle is null.
     * </p>
     */
    public void testGetPathSeparatorStyle() {
        assertNull("Default pathSeparatorStyle should be null.", tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathSeparatorStyle(String) for accuracy.
     * Expects the pathSeparatorStyle is correctly set.
     * </p>
     */
    public void testSetPathSeparatorStyle() {
        tag.setPathSeparatorStyle(STYLE);

        assertEquals("Failed to get correct path separator style.", STYLE, tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathSeparatorStyle(String) for accuracy.
     * It tests the case that when pathSeparatorStyle is null and expects success.
     * </p>
     */
    public void testSetPathSeparatorStyle_NullPathSeparatorStyle() {
        tag.setPathSeparatorStyle(null);

        assertNull("Expects null path separator style.", tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathSeparatorStyle(String) for accuracy.
     * It tests the case that when pathSeparatorStyle is empty and expects success.
     * </p>
     */
    public void testSetPathSeparatorStyle_EmptyPathSeparatorStyle() {
        tag.setPathSeparatorStyle("  ");

        assertNull("Expects null path separator style.", tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getNodeStyle() for accuracy.
     * Expects the default value for nodeStyle is null.
     * </p>
     */
    public void testGetNodeStyle() {
        assertNull("Default node style should be null.", tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setNodeStyle(String) for accuracy.
     * Expecst nodeStyle is correctly set.
     * </p>
     */
    public void testSetNodeStyle() {
        tag.setNodeStyle(STYLE);

        assertEquals("Failed to get correct node style.", STYLE, tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setNodeStyle(String) for accuracy.
     * It tests the case that when nodeStyle is null and expects success.
     * </p>
     */
    public void testSetNodeStyle_NullNodeStyle() {
        tag.setNodeStyle(null);

        assertNull("Expects null node style.", tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setNodeStyle(String) for accuracy.
     * It tests the case that when nodeStyle is empty and expects success.
     * </p>
     */
    public void testSetNodeStyle_EmptyNodeStyle() {
        tag.setNodeStyle("  ");

        assertNull("Expects null node style.", tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getCurrentNodeStyle() for accuracy.
     * Expects the default value for currentNodeStyle is null.
     * </p>
     */
    public void testGetCurrentNodeStyle() {
        assertNull("Default current node style should be null.", tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setCurrentNodeStyle(String) for accuracy.
     * Expects currentNodeStyle is correctly set.
     * </p>
     */
    public void testSetCurrentNodeStyle() {
        tag.setCurrentNodeStyle(STYLE);

        assertEquals("Failed to get correct current node style.", STYLE, tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setCurrentNodeStyle(String) for accuracy.
     * It tests the case that when currentNodeStyle is null and expects success.
     * </p>
     */
    public void testSetCurrentNodeStyle_NullCurrentNodeStyle() {
        tag.setCurrentNodeStyle(null);

        assertNull("Expects null current node style.", tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setCurrentNodeStyle(String) for accuracy.
     * It tests the case that when currentNodeStyle is empty and expects success.
     * </p>
     */
    public void testSetCurrentNodeStyle_EmptyCurrentNodeStyle() {
        tag.setCurrentNodeStyle("  ");

        assertNull("Expects null current node style.", tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getRootNodeStyle() for accuracy.
     * Expects the default value for rootNodeStyle is null.
     * </p>
     */
    public void testGetRootNodeStyle() {
        assertNull("Default root node style should be null.", tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setRootNodeStyle(String) for accuracy.
     * Expects rootNodeStyle is correctly set.
     * </p>
     */
    public void testSetRootNodeStyle() {
        tag.setRootNodeStyle(STYLE);

        assertEquals("Failed to get correct root node style.", STYLE, tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setRootNodeStyle(String) for accuracy.
     * It tests the case that when rootNodeStyle is null and expects success.
     * </p>
     */
    public void testSetRootNodeStyle_NullRootNodeStyle() {
        tag.setRootNodeStyle(null);

        assertNull("Expects null root node style.", tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setRootNodeStyle(String) for accuracy.
     * It tests the case that when rootNodeStyle is empty and expects success.
     * </p>
     */
    public void testSetRootNodeStyle_EmptyRootNodeStyle() {
        tag.setRootNodeStyle("  ");

        assertNull("Expects null root node style.", tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getMouseOverStyle() for accuracy.
     * Expects the default value for mouseOverStyle is null.
     * </p>
     */
    public void testGetMouseOverStyle() {
        assertNull("Default mouse over style should be null.", tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setMouseOverStyle(String) for accuracy.
     * Expects mouseOverStyle is correctly set.
     * </p>
     */
    public void testSetMouseOverStyle() {
        tag.setMouseOverStyle(STYLE);

        assertEquals("Failed to get correct mouse over style.", STYLE, tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setMouseOverStyle(String) for accuracy.
     * It tests the case that when mouseOverStyle is null and expects success.
     * </p>
     */
    public void testSetMouseOverStyle_NullMouseOverStyle() {
        tag.setMouseOverStyle(null);

        assertNull("Expects null mouse over style.", tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setMouseOverStyle(String) for accuracy.
     * It tests the case that when mouseOverStyle is empty and expects success.
     * </p>
     */
    public void testSetMouseOverStyle_EmptyMouseOverStyle() {
        tag.setMouseOverStyle("  ");

        assertNull("Expects null mouse over style.", tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getPathOverride() for accuracy.
     * Expects the default value for pathOverride is null.
     * </p>
     */
    public void testGetPathOverride() {
        assertNull("Default path override should be null.", tag.getPathOverride());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathOverride(String) for accuracy.
     * Expects pathOverride is correctly set.
     * </p>
     */
    public void testSetPathOverride() {
        String xmlStr = "<bc_node title=\"main\" url=\"/main\" pattern=\"/main\"/>";
        tag.setPathOverride(xmlStr);

        assertEquals("Failed to get correct path override.", xmlStr, tag.getPathOverride());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathOverride(String) for accuracy.
     * It tests the case that when pathOverride is null and expects success.
     * </p>
     */
    public void testSetPathOverride_NullPathOverride() {
        tag.setPathOverride(null);

        assertNull("Expects null path override.", tag.getPathOverride());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setPathOverride(String) for accuracy.
     * It tests the case that when pathOverride is empty and expects success.
     * </p>
     */
    public void testSetPathOverride_EmptyPathOverride() {
        tag.setPathOverride("  ");

        assertNull("Expects null path override.", tag.getPathOverride());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#getSilentErrors() for accuracy.
     * Expects the default value for silentErrors is false.
     * </p>
     */
    public void testGetSilentErrors() {
        assertFalse("Default silentErrors should be false", tag.getSilentErrors());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#setSilentErrors(boolean) for accuracy.
     * Expects silentErrors is correctly set.
     * </p>
     */
    public void testSetSilentErrors() {
        tag.setSilentErrors(true);

        assertTrue("Failed to get correct silentErrors.", tag.getSilentErrors());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests the NodeDiscovery and PathDiscovery instances are constructed via Object Factory and expects
     * success.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag() throws Exception {
        TestHelper.loadMultipleXMLConfig(BreadCrumbTrailTag.class.getName(), "test_files/config/config.xml");

        PageContext pageContext = new MockPageContext();
        tag.setPageContext(pageContext);

        assertEquals("Expects that doStartTag returns the int value that EVAL_BODY_BUFFERED represents.",
                BodyTag.EVAL_BODY_BUFFERED, tag.doStartTag());

        TestHelper.clearConfigFile(BreadCrumbTrailTag.class.getName());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests the current node and pathOverride are specified and expects success.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag11() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/main", "Main");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());

        assertEquals("Expects that doStartTag returns the int value that EVAL_BODY_BUFFERED represents.",
                BodyTag.EVAL_BODY_BUFFERED, tag.doStartTag());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case that the current node and pathOverride are specified, styles are set.
     * Expects that doStartTag() executes success and span styles are generated success.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag12() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/main", "Main");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());

        tag.setRootNodeStyle("color:red");
        tag.setNodeStyle("color:red");
        tag.setCurrentNodeStyle("color:red");
        tag.setMouseOverStyle("color:red");
        tag.setPathSeparatorStyle("color:red");

        assertEquals("Expects that doStartTag returns the int value that EVAL_BODY_BUFFERED represents.",
                BodyTag.EVAL_BODY_BUFFERED, tag.doStartTag());

        String styles = deleteWhiteSpace(pageContext.getOut().toString());

        String expectedStyles = "<span><style>.path-separator-style{color:red}a.node-style:link{color:red}"
                + "a.current-node-style:link{color:red}a.root-node-style:link{color:red}a.node-style:hover{color:red}"
                + "a.current-node-style:hover{color:red}a.root-node-style:hover{color:red}</style>";

        assertEquals("Failed to generate correct span styles", expectedStyles, styles);

    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests no shortest path found and expects *SKIP_BODY* returned.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag2() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/text", "TEXT");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setSilentErrors(true);

        assertEquals("Expects that doStartTag returns the int value that SKIP_BODY represents.", BodyTag.SKIP_BODY,
                tag.doStartTag());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests that pathOverride is not valid xml string and expects *SKIP_BODY* returned when silentErrors
     * is true.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag31() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/text", "TEXT");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setSilentErrors(true);

        assertEquals("Expects that doStartTag returns the int value that SKIP_BODY represents.", BodyTag.SKIP_BODY,
                tag.doStartTag());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests that pathOverride is not valid xml string and expects BreadCrumbException when silentErrors
     * is false.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag32() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/text", "TEXT");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride() + "</bc_node>");
        tag.setSilentErrors(false);

        try {
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests that creating the NodeDiscovery instance when Object Factory is not configed.
     * Expects *SKIP_BODY* returned when silentErrors is true.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag41() throws Exception {
        PageContext pageContext = new MockPageContext();

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride() + "</bc_node>");
        tag.setSilentErrors(true);

        assertEquals("Expects that doStartTag returns the int value that SKIP_BODY represents.", BodyTag.SKIP_BODY,
                tag.doStartTag());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests that creating the NodeDiscovery instance when Object Factory is not configed.
     * Expects BreadCrumbException returned when silentErrors is false.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag42() throws Exception {
        PageContext pageContext = new MockPageContext();

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setSilentErrors(false);

        try {
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests that creating the PathDiscovery instance when Object Factory is not configed.
     * Expects *SKIP_BODY* returned when silentErrors is true.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag51() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/text", "TEXT");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setSilentErrors(true);

        assertEquals("Expects that doStartTag returns the int value that SKIP_BODY represents.", BodyTag.SKIP_BODY,
                tag.doStartTag());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doStartTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests that creating the PathDiscovery instance when Object Factory is not configed.
     * Expects BreadCrumbException returned when silentErrors is false.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag52() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/text", "TEXT");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setSilentErrors(false);

        try {
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doInitBody() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when there's only one node in the path and pathDirection is ROOT_TO_CURRENT.
     * Expects BREADCRUMB_FORMAT_NODE and BREADCRUMB_FORMAT_STYLE_CLASS attributes are correctly set.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoInitBody11() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/main", "Main");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);
        tag.doStartTag();

        tag.doInitBody();

        BreadCrumbNode formatNode = (BreadCrumbNode) pageContext.getAttribute(
                BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        String style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", node.getTitle(), formatNode.getTitle());
        assertEquals("Failed to get correct url information.", node.getUrl(), formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "root-node-style", style);
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doInitBody() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when there's only one node in the path and pathDirection is CURRENT_TO_ROOT..
     * Expects BREADCRUMB_FORMAT_NODE and BREADCRUMB_FORMAT_STYLE_CLASS attributes are correctly set.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoInitBody12() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/main", "Main");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);
        tag.doStartTag();

        tag.doInitBody();

        BreadCrumbNode formatNode = (BreadCrumbNode) pageContext.getAttribute(
                BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        String style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", node.getTitle(), formatNode.getTitle());
        assertEquals("Failed to get correct url information.", node.getUrl(), formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "root-node-style", style);
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doInitBody() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when there are there nodes in the path and pathDirection is ROOT_TO_CURRENT.
     * Expects BREADCRUMB_FORMAT_NODE and BREADCRUMB_FORMAT_STYLE_CLASS attributes are correctly set.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoInitBody21() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/topics", "Topics");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);
        tag.doStartTag();

        tag.doInitBody();
        BreadCrumbNode formatNode = (BreadCrumbNode) pageContext.getAttribute(
                BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        String style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", "Main", formatNode.getTitle());
        assertEquals("Failed to get correct url information.", "/main", formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "root-node-style", style);

        tag.doInitBody();
        formatNode = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", "Help", formatNode.getTitle());
        assertEquals("Failed to get correct url information.", "/forums", formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "node-style", style);

        tag.doInitBody();
        formatNode = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", node.getTitle(), formatNode.getTitle());
        assertEquals("Failed to get correct url information.", node.getUrl(), formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "current-node-style", style);
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doInitBody() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when there are there nodes in the path and pathDirection is CURRENT_TO_ROOT.
     * Expects BREADCRUMB_FORMAT_NODE and BREADCRUMB_FORMAT_STYLE_CLASS attributes are correctly set.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoInitBody22() throws Exception {
        PageContext pageContext = new MockPageContext();
        BreadCrumbNode node = new BreadCrumbNode("/topics", "Topics");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPageContext(pageContext);
        tag.setPathOverride(TestHelper.getPathOverride());
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);
        tag.doStartTag();

        tag.doInitBody();
        BreadCrumbNode formatNode = (BreadCrumbNode) pageContext.getAttribute(
                BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        String style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", node.getTitle(), formatNode.getTitle());
        assertEquals("Failed to get correct url information.", node.getUrl(), formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "current-node-style", style);

        tag.doInitBody();
        formatNode = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", "Help", formatNode.getTitle());
        assertEquals("Failed to get correct url information.", "/forums", formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "node-style", style);

        tag.doInitBody();
        formatNode = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);
        style = (String) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertEquals("Failed to get correct title information.", "Main", formatNode.getTitle());
        assertEquals("Failed to get correct url information.", "/main", formatNode.getUrl());
        assertEquals("Failed to get correct style information.", "root-node-style", style);
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doEndTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when no style is set and expects EVAL_PAGE returned.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoEndTag() throws Exception {
        PageContext pageContext = new MockPageContext();
        tag.setPageContext(pageContext);

        tag.setNodeStyle("color:red");

        assertEquals("Expects that doStartTag returns the int value that EVAL_PAGE represents.", BodyTag.EVAL_PAGE,
                tag.doEndTag());

        assertEquals("Failed to get correct string written to jsp writer.", "</span>", pageContext.getOut().toString());
    }

    /**
     * <p>
     * Tests BreadCrumbTrailTag#doEndTag() for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when some style is set and expects EVAL_PAGE returned and
     * &quot;&lt;/span&gt;&quot; written to jspWriter.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoEndTag1() throws Exception {
        PageContext pageContext = new MockPageContext();
        tag.setPageContext(pageContext);

        assertEquals("Expects that doStartTag returns the int value that EVAL_PAGE represents.", BodyTag.EVAL_PAGE,
                tag.doEndTag());
    }

    /**
     * <p>
     * Delete all white spaces in given string.
     * </p>
     *
     * @param s to be used
     *
     * @return new string in which all white spaces are deleted
     */
    private String deleteWhiteSpace(String s) {
        s = s.replaceAll(" ", "");
        s = s.replaceAll("\n", "");
        s = s.replaceAll("\r", "");
        s = s.replaceAll("\t", "");

        return s;
    }
}