/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.el.ExpressionEvaluator;
import javax.servlet.jsp.el.VariableResolver;

/**
 * <p>
 * This class extends PageContext class and it is used for testing.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class MockPageContext extends PageContext {
    /**
     * <p>
     * Stores the attributes in page scope.
     * </p>
     */
    private Map pageAttrs;

    /**
     * <p>
     * Stores the attributes in request scope.
     * </p>
     */
    private Map requestAttrs;

    /**
     * <p>
     * Stores the attributes in session scope.
     * </p>
     */
    private Map sessionAttrs;

    /**
     * <p>
     * Stores the attributes in application scope.
     * </p>
     */
    private Map applicationAttrs;

    /**
     * <p>
     * ServletRequest instance to be returned in getRequest() method.
     * </p>
     *
     */
    private ServletRequest request;

    /**
     * <p>
     * JspWriter instance to be returned in getOut() method.
     * </p>
     */
    private JspWriter jspWriter;

    /**
     * <p>
     * Constructs a MockPageContext instance.
     * </p>
     *
     */
    public MockPageContext() {
        pageAttrs = new HashMap();
        requestAttrs = new HashMap();
        sessionAttrs = new HashMap();
        applicationAttrs = new HashMap();

        jspWriter = new MockJspWriter();
    }

    /**
     * <p>
     * Returns a JspWriter instance for writing.
     * </p>
     *
     * @return a JspWriter instance for writing.
     */
    public JspWriter getOut() {
        return jspWriter;
    }

    /**
     * <p>
     * Implements PageContext#setAttribute(String,Object).
     * </p>
     *
     * @param name attribute name
     * @param value attribute value
     */
    public void setAttribute(String name, Object value) {
        setAttribute(name, value, PageContext.PAGE_SCOPE);
    }

    /**
     * <p>
     * Implements PageContext#setAttribute(String,Object,int).
     * </p>
     *
     * @param name attribute name
     * @param value attribute value
     * @param scope scope value
     *
     * @throws IllegalArgumentException if an invalid scope value is passed-in.
     */
    public void setAttribute(String name, Object value, int scope) {
        if (scope == PageContext.PAGE_SCOPE) {
            pageAttrs.put(name, value);
        } else if (scope == PageContext.REQUEST_SCOPE) {
            requestAttrs.put(name, value);
        } else if (scope == PageContext.SESSION_SCOPE) {
            sessionAttrs.put(name, value);
        } else if (scope == PageContext.APPLICATION_SCOPE) {
            applicationAttrs.put(name, value);
        } else {
            throw new IllegalArgumentException("Invalid scope " + scope);
        }
    }

    /**
     * <p>
     * Implements PageContext#getAttribute(String).
     * </p>
     *
     * @param name attribute name
     *
     * @return the attribute value
     */
    public Object getAttribute(String name) {
        return getAttribute(name, PageContext.PAGE_SCOPE);
    }

    /**
     * <p>
     * Implements PageContext#getAttribute(String,int).
     * </p>
     *
     * @param name attribute name
     * @param scope attribute scope value
     * @return attribute value
     *
     * @throws IllegalArgumentException if an invalid scope value is passed-in.
     */
    public Object getAttribute(String name, int scope) {
        if (scope == PageContext.PAGE_SCOPE) {
            return pageAttrs.get(name);
        } else if (scope == PageContext.REQUEST_SCOPE) {
            return requestAttrs.get(name);
        } else if (scope == PageContext.SESSION_SCOPE) {
            return sessionAttrs.get(name);
        } else if (scope == PageContext.APPLICATION_SCOPE) {
            return applicationAttrs.get(name);
        } else {
            throw new IllegalArgumentException("Invalid scope " + scope);
        }
    }

    /**
     * <p>
     * Implements PageContext#getAttribute(String).
     * </p>
     *
     * @param name attribute name
     * @return attribute value
     */
    public Object findAttribute(String name) {
        Object value = getAttribute(name, PageContext.PAGE_SCOPE);
        if (value == null) {
            value = getAttribute(name, PageContext.REQUEST_SCOPE);
        }
        if (value == null) {
            value = getAttribute(name, PageContext.SESSION_SCOPE);
        }
        if (value == null) {
            value = getAttribute(name, PageContext.APPLICATION_SCOPE);
        }
        return value;
    }

    /**
     * <p>
     * Implements PageContext#removeAttribute(String).
     * </p>
     *
     * @param name attribute name
     */
    public void removeAttribute(String name) {
        int scope = getAttributesScope(name);
        if (scope != 0) {
            removeAttribute(name, scope);
        }
    }

    /**
     * <p>
     * Implements PageContext#removeAttribute(String,int).
     * </p>
     *
     * @param name attribute name
     * @param scope attribute scope value
     *
     * @throws IllegalArgumentException if an invalid scope value is passed-in.
     */
    public void removeAttribute(String name, int scope) {
        if (scope == PageContext.PAGE_SCOPE) {
            pageAttrs.remove(name);
        } else if (scope == PageContext.REQUEST_SCOPE) {
            requestAttrs.remove(name);
        } else if (scope == PageContext.SESSION_SCOPE) {
            sessionAttrs.remove(name);
        } else if (scope == PageContext.APPLICATION_SCOPE) {
            applicationAttrs.remove(name);
        } else {
            throw new IllegalArgumentException("Invalid scope " + scope);
        }

    }

    /**
     * <p>
     * Implements PageContext#getAttributesScope(String).
     * </p>
     *
     * @param name attribute name
     *
     * @return attribute scope value
     */
    public int getAttributesScope(String name) {
        if (getAttribute(name, PageContext.PAGE_SCOPE) != null) {
            return PageContext.PAGE_SCOPE;
        } else if (getAttribute(name, PageContext.REQUEST_SCOPE) != null) {
            return PageContext.REQUEST_SCOPE;
        } else if (getAttribute(name, PageContext.SESSION_SCOPE) != null) {
            return PageContext.SESSION_SCOPE;
        } else if (getAttribute(name, PageContext.APPLICATION_SCOPE) != null) {
            return PageContext.APPLICATION_SCOPE;
        }
        return 0;

    }

    /**
     * <p>
     * Returns a ServletRequest instance.
     * </p>
     *
     * @return a ServletRequest instance
     */
    public ServletRequest getRequest() {
        return request;
    }

    /**
     * <p>
     * Sets the ServletRequest instance.
     * </p>
     *
     * @param request ServletRequest instance
     */
    public void setRequest(ServletRequest request) {
        this.request = request;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param servlet servlet
     * @param request request
     * @param response response
     * @param errorPageURL errorPageURL
     * @param needsSession needsSession
     * @param bufferSize bufferSize
     * @param autoFlush autoFlush
     */
    public void initialize(Servlet servlet, ServletRequest request, ServletResponse response, String errorPageURL,
            boolean needsSession, int bufferSize, boolean autoFlush) {
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     */
    public void release() {
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public HttpSession getSession() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public Object getPage() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public ServletResponse getResponse() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public Exception getException() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public ServletConfig getServletConfig() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public ServletContext getServletContext() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param path path
     */
    public void forward(String path) {
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param path path
     */
    public void include(String path) {

    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param path path
     * @param flush flush
     */
    public void include(String path, boolean flush) {
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param exc exc
     */
    public void handlePageException(Exception exc) {
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param thr thr
     */
    public void handlePageException(Throwable thr) {
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public ExpressionEvaluator getExpressionEvaluator() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @return null
     */
    public VariableResolver getVariableResolver() {
        return null;
    }

    /**
     * <p>
     * Not implemented.
     * </p>
     *
     * @param scope scope
     * @return null
     */
    public Enumeration getAttributeNamesInScope(int scope) {
        return null;
    }

    /**
     * <p>
     * This class extends JspWriter and is used for testing.
     * </p>
     *
     * @author TCSDEVELOPER
     * @version 2.0
     *
     */
    private class MockJspWriter extends JspWriter {
        /**
         * <p>
         * The underlying writer that all the calls to this JspWriter will be redirected to.
         * </p>
         */
        private StringWriter writer;

        /**
         * <p>
         * Wrap the StringWriter as a PrintWriter.
         * </p>
         */
        private PrintWriter printWriter;

        /**
         * <p>
         * Consturcts a MockJspWriter instance.
         * </p>
         *
         */
        protected MockJspWriter() {
            super(0, true);
            writer = new StringWriter();
            printWriter = new PrintWriter(writer);
        }

        /**
         * <p>
         * Returns the content that writes to the underlying writer.
         * </p>
         *
         * @return the content that writes to the underlying writer.
         */
        public String toString() {
            return writer.toString();
        }

        /**
         * <p>
         * Clears the output.
         * </p>
         *
         * @throws IOException if an I/O error occurs.
         */
        public void clear() throws IOException {
            writer = new StringWriter();
            printWriter = new PrintWriter(writer);
        }

        /**
         * <p>
         * Clears the output.
         * </p>
         *
         * @throws IOException if an I/O error occurs.
         */
        public void clearBuffer() throws IOException {
            clear();
        }

        /**
         * <p>
         * Implements JspWriter#close().
         * </p>
         *
         * @throws IOException if an I/O error occurs.
         */
        public void close() throws IOException {
            flush();
            printWriter.close();
        }

        /**
         * <p>
         * Implements JspWriter#getRemaining().
         * </p>
         *
         * @return 0
         */
        public int getRemaining() {
            return 0;
        }

        /**
         * <p>
         * Implements JspWriter#flush().
         * </p>
         *
         * @throws IOException if an I/O error occurs.
         */
        public void flush() throws IOException {
            printWriter.flush();
        }

        /**
         * <p>
         * Implements JspWriter#newLine().
         * </p>
         *
         * @throws IOException if an I/O error occurs.
         */
        public void newLine() throws IOException {
            print("\r\n");
        }

        /**
         * <p>
         * Implements JspWriter#print(boolean).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(boolean value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(char).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(char value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(char[]).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(char[] value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(double).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(double value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(float).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(float value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(int).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(int value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(long).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(long value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(Object).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(Object value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#print(String).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void print(String value) throws IOException {
            printWriter.print(value);
        }

        /**
         * <p>
         * Implements JspWriter#println().
         * </p>
         *
         * @throws IOException if an I/O error occurs.
         */
        public void println() throws IOException {
            printWriter.println();
        }

        /**
         * <p>
         * Implements JspWriter#println(boolean).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(boolean value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(char).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(char value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(char[]).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(char[] value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(double).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(double value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(float).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(float value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(int).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(int value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(long).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(long value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(Object).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(Object value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#println(String).
         * </p>
         *
         * @param value value
         * @throws IOException if an I/O error occurs.
         */
        public void println(String value) throws IOException {
            printWriter.println(value);
        }

        /**
         * <p>
         * Implements JspWriter#write().
         * </p>
         *
         * @param cbuf cbuf
         * @param off off
         * @param len len
         *
         * @throws IOException if an I/O error occurs.
         */
        public void write(char[] cbuf, int off, int len) throws IOException {
            printWriter.write(cbuf, off, len);
        }
    }
}
