/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.Tag;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

/**
 * <p>
 * In-container test cases for HttpServletNodeDiscoveryTag and InlineTextNodeDiscoveryTag.
 * </p>
 *
 * <p>
 * For unit test cases, please see HttpServletNodeDiscoveryTagTests and InlineTextNodeDiscoveryTagTests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 *
 */
public class NodeDiscoveryTagInContainerTests extends JspTestCase {
    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(NodeDiscoveryTagInContainerTests.class);
    }

    /**
     * <p>
     * In-container test for HttpServletNodeDiscoveryTag.
     * </p>
     *
     * <p>
     * Expects the node is successfully created according to the ServerletRequest information.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testHttpServeletNDT() throws Exception {
        HttpServletNodeDiscoveryTag tag = new HttpServletNodeDiscoveryTag();

        tag.setPageContext(pageContext);
        tag.setTitle("Main");

        int value = tag.doStartTag();

        assertEquals("Failed to return the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);

        Object obj = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, PageContext.PAGE_SCOPE);

        assertTrue("The attribute value should be of BreadCrumbNode type.", obj instanceof BreadCrumbNode);

        BreadCrumbNode node = (BreadCrumbNode) obj;
        assertEquals("Failed to get correct title.", "Main", node.getTitle());

        String url = "/bread_crumb_trail_tag-test/JspRedirector?Cactus_TestMethod=testHttpServeletNDT&"
                + "Cactus_TestClass=com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery."
                + "NodeDiscoveryTagInContainerTests&Cactus_AutomaticSession=true&Cactus_Service=CALL_TEST";
        assertEquals("Failed to get correct url.", url, node.getUrl());
    }

    /**
     * <p>
     * In-container test for InlineTextNodeDiscoveryTag.
     * </p>
     *
     * <p>
     * Expects the node is successfully created according to the tag information.
     * </p>
     *
     *
     * @throws Exception to JUnit
     */
    public void testInlineTextNDT() throws Exception {
        InlineTextNodeDiscoveryTag tag = new InlineTextNodeDiscoveryTag();

        tag.setPageContext(pageContext);
        tag.setTitle("TopCoder");

        BodyContent bodyContent = pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        bodyContent.print("/tc");

        int value = tag.doAfterBody();

        assertEquals("Failed to return the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);

        Object obj = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, PageContext.PAGE_SCOPE);

        assertTrue("The attribute value should be of BreadCrumbNode type.", obj instanceof BreadCrumbNode);

        BreadCrumbNode node = (BreadCrumbNode) obj;
        assertEquals("Failed to get correct title.", "TopCoder", node.getTitle());
        assertEquals("Failed to get correct url.", "/tc", node.getUrl());

        pageContext.popBody();
    }

}
