/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import javax.servlet.jsp.PageContext;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.MockPageContext;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for HtmlAnchorNodeFormatterTag.
 * </p>
 *
 * <p>
 * For in-container tests, please see NodeFormatterTagInContainerTests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class HtmlAnchorNodeFormatterTagTests extends TestCase {
    /**
     * <p>
     * HtmlAnchorNodeFormatterTag instance for testing.
     * </p>
     */
    private HtmlAnchorNodeFormatterTag tag;

    /**
     * <p>
     * PageContext instance for testing.
     * </p>
     */
    private PageContext pageContext;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        pageContext = new MockPageContext();
        tag = new HtmlAnchorNodeFormatterTag();

        tag.setPageContext(pageContext);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        tag = null;
        pageContext = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(HtmlAnchorNodeFormatterTagTests.class);
    }

    /**
     * <p>
     * Tests ctor HtmlAnchorNodeFormatterTag#HtmlAnchorNodeFormatterTag() for accuracy.
     * Expects tag is successfully created.
     * </p>
     */
    public void testCtor() {
        assertNotNull("Failed to create new HtmlAnchorNodeFormatterTag instance.", tag);
    }

    /**
     * <p>
     * Tests HtmlAnchorNodeFormatterTag#doStartTag() for failure.
     * </p>
     *
     * <p>
     * It tests the case when BREADCRUMB_FORMAT_NODE attribute is not specified in pageContext and
     * expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoStartTag_BreadCrumbException() throws Exception {
        try {
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests HtmlAnchorNodeFormatterTag#doStartTag() for failure.
     * </p>
     *
     * <p>
     * It tests the case when BREADCRUMB_FORMAT_NODE attribute is not of BreadCrumbNode type in pageContext and
     * expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoStartTag_BreadCrumbException2() throws Exception {
        try {
            pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "node");
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests HtmlAnchorNodeFormatterTag#doStartTag() for failure.
     * </p>
     *
     * <p>
     * It tests the case when BREADCRUMB_FORMAT_STYLE_CLASS attribute is not of String type in pageContext and
     * expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoStartTag_BreadCrumbException3() throws Exception {
        try {
            pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, new StringBuffer("node"));
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }
}