/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import javax.servlet.jsp.tagext.Tag;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebResponse;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

/**
 * <p>
 * In-container test cases for HtmlAnchorNodeFormatterTag and TemplateNodeFormatterTag.
 * </p>
 *
 * <p>
 * For unit test cases, please see HtmlAnchorNodeFormatterTagTests and TemplateNodeFormatterTagTests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 *
 */
public class NodeFormatterTagInContainerTests extends JspTestCase {
    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(NodeFormatterTagInContainerTests.class);
    }

    /**
     * <p>
     * In-container test for HttpServletNodeDiscoveryTag.
     * </p>
     *
     * <p>
     * It tests the case when no style is specified.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testHtmlAnchorNFT_NoStyle() throws Exception {
        HtmlAnchorNodeFormatterTag tag = new HtmlAnchorNodeFormatterTag();
        tag.setPageContext(pageContext);

        BreadCrumbNode node = new BreadCrumbNode("http://www.topcoder.com", "TOPCODER");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);

        int value = tag.doStartTag();

        assertEquals("Failed to return the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endHtmlAnchorNFT_NoStyle(WebResponse response) {
        String expectedHtml = "<a href=\"http://www.topcoder.com\">TOPCODER</a>";
        assertEquals("Failed to create correct html anchor.", expectedHtml, response.getText());
    }

    /**
     * <p>
     * In-container test for HttpServletNodeDiscoveryTag.
     * </p>
     *
     * <p>
     * It tests the case when style is specified.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testHtmlAnchorNFT_WithStyle() throws Exception {
        HtmlAnchorNodeFormatterTag tag = new HtmlAnchorNodeFormatterTag();
        tag.setPageContext(pageContext);

        BreadCrumbNode node = new BreadCrumbNode("http://www.topcoder.com", "TOPCODER");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "node-style");

        int value = tag.doStartTag();

        assertEquals("Failed to return the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endHtmlAnchorNFT_WithStyle(WebResponse response) {
        String expectedHtml = "<a class=\"node-style\" href=\"http://www.topcoder.com\">TOPCODER</a>";
        assertEquals("Failed to create correct html anchor.", expectedHtml, response.getText());
    }

    /**
     * <p>
     * In-container test for HttpServletNodeDiscoveryTag.
     * </p>
     *
     * <p>
     * It tests the case when the current node has null title.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testHtmlAnchorNFT_NullTitle() throws Exception {
        HtmlAnchorNodeFormatterTag tag = new HtmlAnchorNodeFormatterTag();
        tag.setPageContext(pageContext);

        BreadCrumbNode node = new BreadCrumbNode("http://www.topcoder.com", null);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);

        int value = tag.doStartTag();

        assertEquals("Failed to return the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endHtmlAnchorNFT_NullTitle(WebResponse response) {
        String expectedHtml = "<a href=\"http://www.topcoder.com\">Unspecified</a>";
        assertEquals("Failed to create correct html anchor.", expectedHtml, response.getText());
    }

    /**
     * <p>
     * In-container test for TemplateNodeFormatterTag.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testTemplateNodeFormatterTage() throws Exception {
        TemplateNodeFormatterTag tag = new TemplateNodeFormatterTag("template.jsp");
        tag.setPageContext(pageContext);

        BreadCrumbNode node = new BreadCrumbNode("http://www.topcoder.com", "TOPCODER");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "node-style");

        int value = tag.doStartTag();

        assertEquals("Failed to return the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endTemplateNodeFormatterTage(WebResponse response) {
        String expectedHtml = "<a href=\"http://www.topcoder.com\">TOPCODER</a>";
        assertEquals("Failed to create correct template anchor.", expectedHtml, response.getText().trim());
    }
}
