/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.PageContext;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.MockPageContext;
import com.topcoder.web.ui.tag.breadcrumbtrail.TestHelper;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for XmlPathDiscovery.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class XmlPathDiscoveryTests extends TestCase {

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(XmlPathDiscoveryTests.class);
    }

    /**
     * <p>
     * Tests ctor XmlPathDiscovery#XmlPathDiscovery(String,boolean) for accuracy.
     * Expects the XmlPathDiscovery instance is created successfully.
     * </p>
     */
    public void testCtor1() {
        String bcNode = "<BreadCrumb><bc_node title=\"title root\" url=\"/title_root\" pattern=\"/title_root\"/>"
                + "</BreadCrumbNode>";
        assertNotNull("Failed to create new XmlPathDiscovery instance.", new XmlPathDiscovery(bcNode, false));
    }

    /**
     * <p>
     * Tests ctor XmlPathDiscovery#XmlPathDiscovery(String,boolean) for failure.
     * It tests the case that when source is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_NullSource() {
        try {
            new XmlPathDiscovery(null, true);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlPathDiscovery#XmlPathDiscovery(String,boolean) for failure.
     * It tests the case that when source is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_EmptySource() {
        try {
            new XmlPathDiscovery("  ", true);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlPathDiscovery#XmlPathDiscovery(String) for accuracy.
     * Expects the XmlPathDiscovery instance is created successfully.
     * </p>
     */
    public void testCtor2() {
        assertNotNull("Failed to create new XmlPathDiscovery instance.", new XmlPathDiscovery(
                "test_files/siteMap/SimpleSiteMap.xml"));
    }

    /**
     * <p>
     * Tests ctor XmlPathDiscovery#XmlPathDiscovery(String) for failure.
     * It tests the case that when filename is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor2_NullFilename() {
        try {
            new XmlPathDiscovery(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlPathDiscovery#XmlPathDiscovery(String) for failure.
     * It tests the case that when filename is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor2_EmptyFilename() {
        try {
            new XmlPathDiscovery(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getPath(BreadCrumbNode,PageContext) for accuracy.
     * </p>
     *
     * <p>
     * It tests the case before reaching the terminal node, a loop occurs.
     * </p>
     *
     * <p>
     * The siteMap graph:
     * &lt;bc_node title=&quot;bcn1&quot; url=&quot;/bcn1&quot; pattern=&quot;/bcn1&quot;&gt;
     *     &lt;bc_node title=&quot;bcn2&quot; url=&quot;/bcn2&quot; pattern=&quot;/bcn2&quot;&gt;
     *         &lt;bc_node title=&quot;bcn3&quot; url=&quot;/bcn3&quot; pattern=&quot;/bcn3&quot;&gt;
     *             &lt;bc_node title=&quot;bcn4&quot; url=&quot;/bcn4&quot; pattern=&quot;/bcn4&quot; /&gt;
     *         &lt;/bc_node&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     *
     * &lt;bc_node title=&quot;bcn4&quot; url=&quot;/bcn4&quot; pattern=&quot;/bcn4&quot;&gt;
     *     &lt;bc_node title=&quot;bcn2&quot; url=&quot;/bcn2&quot; pattern=&quot;/bcn2&quot;/&gt;
     *     &lt;bc_node title=&quot;bcn5&quot; url=&quot;/bcn5&quot; pattern=&quot;/bcn5&quot;&gt;
     *         &lt;bc_node title=&quot;bcn3&quot; url=&quot;/bcn3&quot; pattern=&quot;/bcn3&quot;/&gt;
     *         &lt;bc_node title=&quot;bcn6&quot; url=&quot;/bcn6&quot; pattern=&quot;/bcn6&quot;/&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetPath() throws Exception {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/XmlPathNodeDiscoverySiteMap.xml");
        PageContext pageContext = new MockPageContext();

        BreadCrumbNode node = new BreadCrumbNode("/bcn6", "bcn6");

        BreadCrumbNode[] actualNodes = pathDiscovery.getPath(node, pageContext);

        BreadCrumbNode[] expectedNodes = new BreadCrumbNode[6];
        expectedNodes[0] = new BreadCrumbNode("/bcn1", "bcn1");
        expectedNodes[1] = new BreadCrumbNode("/bcn2", "bcn2");
        expectedNodes[2] = new BreadCrumbNode("/bcn3", "bcn3");
        expectedNodes[3] = new BreadCrumbNode("/bcn4", "bcn4");
        expectedNodes[4] = new BreadCrumbNode("/bcn5", "bcn5");
        expectedNodes[5] = new BreadCrumbNode("/bcn6", "bcn6");

        assertTrue("Failed to get correct bread crumb trail.", TestHelper.isBreadCrumbNodeArrayEquals(expectedNodes,
                actualNodes));
        assertNotNull("Expects *CURRENT_SITEMAP* attribute be set.", pageContext.getAttribute(
                XmlPathDiscovery.CURRENT_SITEMAP, PageContext.SESSION_SCOPE));
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getPath(BreadCrumbNode,PageContext) for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when the current node has null title and expected the title is replaced on the returned path.
     * </p>
     *
     * <p>
     * The siteMap graph:
     * &lt;bc_node title=&quot;bcn1&quot; url=&quot;/bcn1&quot; pattern=&quot;/bcn1&quot;&gt;
     *     &lt;bc_node title=&quot;bcn2&quot; url=&quot;/bcn2&quot; pattern=&quot;/bcn2&quot;&gt;
     *         &lt;bc_node title=&quot;bcn4&quot; url=&quot;/bcn4&quot; pattern=&quot;/bcn[?].*&quot;/&gt;
     *     &lt;/bc_node&gt;
     *     &lt;bc_node title=&quot;bcn3&quot; url=&quot;/bcn3&quot; pattern=&quot;/bcn3&quot;&gt;
     *         &lt;bc_node title=&quot;bcn5&quot; url=&quot;/bcn5&quot; pattern=&quot;/bcn[?].*&quot;/&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetPath_TitleReplaced() throws Exception {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/XmlPathNodeDiscoverySiteMap1.xml");
        PageContext pageContext = new MockPageContext();

        BreadCrumbNode node = new BreadCrumbNode("/bcn?id=88", null);

        BreadCrumbNode[] actualNodes = pathDiscovery.getPath(node, pageContext);

        BreadCrumbNode[] expectedNodes = new BreadCrumbNode[3];
        expectedNodes[0] = new BreadCrumbNode("/bcn1", "bcn1");
        expectedNodes[1] = new BreadCrumbNode("/bcn2", "bcn2");
        expectedNodes[2] = new BreadCrumbNode("/bcn?id=88", "bcn4");

        assertTrue("Failed to get correct bread crumb trail.", TestHelper.isBreadCrumbNodeArrayEquals(expectedNodes,
                actualNodes));
        assertNotNull("Expects *CURRENT_SITEMAP* attribute be set.", pageContext.getAttribute(
                XmlPathDiscovery.CURRENT_SITEMAP, PageContext.SESSION_SCOPE));
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getPath(BreadCrumbNode,PageContext) for failure.
     * It tests the case that when currentNode is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetPath_NullCurrentNode() throws Exception {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/SimpleSiteMap.xml");
        try {
            pathDiscovery.getPath(null, new MockPageContext());
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getPath(BreadCrumbNode,PageContext) for failure.
     * It tests the case that when pageContext is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetPath_NullPageContext() throws Exception {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/SimpleSiteMap.xml");
        try {
            pathDiscovery.getPath(new BreadCrumbNode("/main", "Main"), null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getSiteMap() for accuracy.
     * Expects site map is set up successfully.
     * </p>
     *
     * <p>
     * The siteMap graph:
     * &lt;bc_node title=&quot;title root&quot; url=&quot;/title_root&quot; pattern=&quot;/title_root&quot;&gt;
     *   &lt;bc_node title=&quot;title Branch 1&quot; url=&quot;/urlbranch1&quot;
     *   pattern=&quot;/urlbranch1?.*&quot;/&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetSiteMap() throws Exception {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/SimpleSiteMap.xml");

        NodeMatcher rootNodeMatcher = new RegexNodeMatcher("title root", "/title_root", "/title_root");
        NodeMatcher childNodeMatcher = new RegexNodeMatcher("title Branch 1", "/urlbranch1", "/urlbranch1?.*");

        Map siteMap = new HashMap();

        List children = new LinkedList();
        children.add(childNodeMatcher);

        siteMap.put(rootNodeMatcher, children);
        siteMap.put(childNodeMatcher, new LinkedList());

        assertEquals("Failed to return correct siteMap.", siteMap, pathDiscovery.getSiteMap());
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getSiteMap() for accuracy.
     * Expects site map is set up successfully.
     * </p>
     *
     * <p>
     * The siteMap graph:
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Forums&quot; url=&quot;/forums&quot; pattern=&quot;/forums&quot;&gt;
     *         &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot;
     *         pattern=&quot;/forums?id.*&quot;/&gt;
     *     &lt;/bc_node&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     * &lt;/bc_node&gt;
     *
     * &lt;bc_node title=&quot;AltMain&quot; url=&quot;/main&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums?id.*&quot;/&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetSiteMap2() throws Exception {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/SiteMap.xml");

        NodeMatcher myMainNodeMatcher = new RegexNodeMatcher("MyMain", "/", "/");
        NodeMatcher forumsNodeMatcher = new RegexNodeMatcher("Forums", "/forums", "/forums");
        NodeMatcher helpNodeMatcher = new RegexNodeMatcher("Help", "/forums", "/forums[?]id.*");
        NodeMatcher topicsNodeMatcher = new RegexNodeMatcher("Topics", "/topics", "/topics");

        NodeMatcher altMainNodeMatcher = new RegexNodeMatcher("AltMain", "/main", "/");

        Map siteMap = new HashMap();

        List myMainChildren = new LinkedList();
        myMainChildren.add(forumsNodeMatcher);
        myMainChildren.add(topicsNodeMatcher);

        List forumsChildren = new LinkedList();
        forumsChildren.add(helpNodeMatcher);

        List altMainChildren = new LinkedList();
        altMainChildren.add(helpNodeMatcher);

        siteMap.put(myMainNodeMatcher, myMainChildren);
        siteMap.put(forumsNodeMatcher, forumsChildren);
        siteMap.put(helpNodeMatcher, new LinkedList());
        siteMap.put(topicsNodeMatcher, new LinkedList());

        siteMap.put(altMainNodeMatcher, altMainChildren);

        assertEquals("Failed to return correct siteMap.", siteMap, pathDiscovery.getSiteMap());
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getSiteMap() for failure.
     * It tests the case when both of the two equal NodeMatcher nodes have children and expects
     * BreadCrumbException.
     * </p>
     *
     * <p>
     * The site map graph:
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Forums&quot; url=&quot;/forums&quot; pattern=&quot;/forums&quot;&gt;
     *        &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;&gt;
     *            &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     *        &lt;/bc_node&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     *
     * &lt;bc_node title=&quot;AltMain&quot; url=&quot;/main&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;&gt;
     *         &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     */
    public void testGetSiteMap_BreadCrumbException1() {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/BadSiteMap1.xml");
        try {
            pathDiscovery.getSiteMap();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getSiteMap() for failure.
     * It tests the case when one NodeMatcher node contains two equal children and expects BreadCrumbException.
     * </p>
     *
     * <p>
     * The site map :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Forums&quot; url=&quot;/forums&quot; pattern=&quot;/forums&quot;&gt;
     *        &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;&gt;
     *            &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     *        &lt;/bc_node&gt;
     *        &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     */
    public void testGetSiteMap_BreadCrumbException2() {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/BadSiteMap2.xml");
        try {
            pathDiscovery.getSiteMap();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlPathDiscovery#getSiteMap() for failure.
     * It tests the case when url attribute is missing and expects BreadCrumbException.
     * </p>
     *
     * <p>
     * &lt;bc_node title=&quot;MyMain&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Forums&quot; url=&quot;/forums&quot; pattern=&quot;/forums&quot;&gt;
     *        &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;&gt;
     *            &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     *        &lt;/bc_node&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     */
    public void testGetSiteMap_BreadCrumbException3() {
        XmlPathDiscovery pathDiscovery = new XmlPathDiscovery("test_files/siteMap/BadSiteMap3.xml");
        try {
            pathDiscovery.getSiteMap();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }
}