/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for RegexNodeMatcher.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class RegexNodeMatcherTests extends TestCase {
    /**
     * <p>
     * Title string for creating new RegexNodeMatcher instance.
     * </p>
     */
    private static final String TITLE = "TOPCODER";

    /**
     * <p>
     * URL String for creating new RegexNodeMatcher instance.
     * </p>
     */
    private static final String URL = "http://www.topcoder.com";

    /**
     * <p>
     * URL_PATTERN String for creating new RegexNodeMatcher instance.
     * </p>
     */
    private static final String URL_PATTERN = "http://www.topcoder.com/.*";

    /**
     * <p>
     * RegexNodeMatcher instance for testing.
     * </p>
     */
    private RegexNodeMatcher nodeMatcher;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void setUp() throws Exception {
        nodeMatcher = new RegexNodeMatcher(TITLE, URL, URL_PATTERN);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        nodeMatcher = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(RegexNodeMatcherTests.class);
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for accuracy.
     * Expects nodeMatcher is successfully created.
     * </p>
     */
    public void testCtor() {
        assertNotNull("Failed to create a new RegexNodeMatcher instance.", nodeMatcher);
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case that when title is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testCtor_NullTitle() throws Exception {
        try {
            new RegexNodeMatcher(null, URL, URL_PATTERN);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case that when title is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testCtor_EmptyTitle() throws Exception {
        try {
            new RegexNodeMatcher(" ", URL, URL_PATTERN);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case that when url is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testCtor_NullUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, null, URL_PATTERN);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case that when url is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testCtor_EmptyUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, " ", URL_PATTERN);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case that when nodePattern is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testCtor_NullNodePattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case that when nodePattern is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testCtor_EmptyNodePattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, "  ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor RegexNodeMatcher#RegexNodeMatcher(String,String,String) for failure.
     * It tests the case when nodePattern is not a valid regex expression and expects for BreadCrumbException.
     * </p>
     */
    public void testCtor_BreadCrumbException() {
        try {
            new RegexNodeMatcher(TITLE, URL, "http://www.top(.*]");
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#hashCode() for accuracy.
     * Expects correct hashcode is returned.
     * </p>
     */
    public void testHashCode() {
        int rightcode = URL_PATTERN.hashCode() + 29 ^ TITLE.hashCode() + 29 ^ URL.hashCode();

        assertEquals("Failed to get correct hashcode", rightcode, nodeMatcher.hashCode());
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#equals(Object) for accuracy.
     * Expects return true when the two RegexNodeMatcher instance are equal.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testEquals() throws Exception {
        RegexNodeMatcher other = new RegexNodeMatcher(TITLE, URL, URL_PATTERN);

        assertTrue("Failed to return true.", nodeMatcher.equals(other));
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#equals(Object) for accuracy.
     * It tests the case that when obj is null and expects false.
     * </p>
     */
    public void testEquals_NullObj() {
        assertFalse("Failed to return false compared to null.", nodeMatcher.equals(null));
    }

    /**
     * <p>
     * Test equals with other type object.
     * Verify that false should be returned.
     * </p>
     */
    public void testEquals_OtherType() {
        assertFalse("Failed to return false while other type object.", nodeMatcher.equals(TITLE));
    }

    /**
     * <p>
     * Test equals with same type , different title.
     * Verify that false should be returned.
     * </p>
     *
     * @throws Exception to JUnit.
     *
     */
    public void testEquals_SameTypeOtherTitle() throws Exception {
        RegexNodeMatcher other = new RegexNodeMatcher("TC", URL, URL_PATTERN);
        assertFalse("Failed to return false while same type , different title.", nodeMatcher.equals(other));
    }

    /**
     * <p>
     * Test equals with same type , different url.
     * Verify that false should be returned.
     * </p>
     *
     * @throws Exception to JUnit.
     *
     */
    public void testEquals_SameTypeOtherURL() throws Exception {
        RegexNodeMatcher other = new RegexNodeMatcher(TITLE, "http://www.tc.com", URL_PATTERN);
        assertFalse("Failed to return false while same type , different url.", nodeMatcher.equals(other));
    }

    /**
     * <p>
     * Test equals with same type , different url pattern.
     * Verify that false should be returned.
     * </p>
     *
     * @throws Exception to JUnit.
     *
     */
    public void testEquals_SameTypeOtherURLPattern() throws Exception {
        RegexNodeMatcher other = new RegexNodeMatcher(TITLE, URL, "http://www.tc.com/.*");
        assertFalse("Failed to return false while same type , different url pattern.", nodeMatcher.equals(other));
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#matches(BreadCrumbNode) for accuracy.
     * It tests the case when the node matches the RegexNodeMatcher instance and expects true be returned.
     * </p>
     */
    public void testMatches() {
        BreadCrumbNode node = new BreadCrumbNode("http://www.topcoder.com/tc", "Main");

        assertTrue("Failed to return true while the nodeMathcer matches the node.", nodeMatcher.matches(node));
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#matches(BreadCrumbNode) for accuracy.
     * It tests the case when the node doesn't match the RegexNodeMatcher instance and expects false be returned.
     * </p>
     */
    public void testMatches2() {
        BreadCrumbNode node = new BreadCrumbNode("http://www.google.com", "Google");

        assertFalse("Failed to return false while the nodeMathcer doesn't match the node.", nodeMatcher.matches(node));
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#matches(BreadCrumbNode) for failure.
     * It tests the case that when node is null and expects false.
     * </p>
     */
    public void testMatches_NullNode() {
        assertFalse("Failed to return false while matching null", nodeMatcher.matches(null));
    }

    /**
     * <p>
     * Tests RegexNodeMatcher#getNodePattern() for accuracy.
     * Expects nodePattern is correctly set in constructor.
     * </p>
     */
    public void testGetNodePattern() {
        assertEquals("Failed to get correct url pattern.", URL_PATTERN, nodeMatcher.getNodePattern());
    }

}